@@ -28,33 +28,38 @@
 #include "glib-utils.h"
 #include "goo-stock.h"
 
-#define SPACING 6
+#define SPACING 0
 #define TITLE1_FORMAT "<span size='large' weight='bold'>%s</span>"
 #define TITLE2_FORMAT "%s"
 #define TITLE3_FORMAT "<i>%s</i>"
 #define TIME_FORMAT "%s"
-#define PLAYING_FORMAT "%s"
+#define PLAYING_FORMAT "[ <small>%s</small> ]"
 #define SCALE_WIDTH 150
 #define COVER_SIZE 80
 #define MIN_WIDTH 400
+#define UPDATE_TIMEOUT 50
 
 struct _GooPlayerInfoPrivateData {
-	GooPlayer *player;
-	GtkWidget *title1_label;
-	GtkWidget *title2_label;
-	GtkWidget *title3_label;
-	GtkWidget *time_label;
-	GtkWidget *playing_label;
-	GtkWidget *time_scale;
-	GtkWidget *image;
-	char       current_time[64];
-	char       total_time[64];
-	char       time[64];
-	gint64     song_length;
-	gboolean   update_slider;
+	GooPlayer   *player;
+	GtkWidget   *title1_label;
+	GtkWidget   *title2_label;
+	GtkWidget   *title3_label;
+	GtkWidget   *time_label;
+	GtkWidget   *playing_label;
+	GtkWidget   *time_scale;
+	GtkWidget   *cover_image;
+	GtkWidget   *cover_button;
+	GtkTooltips *tips;
+	char         current_time[64];
+	char         total_time[64];
+	char         time[64];
+	gint64       song_length;
+	gboolean     dragging;
+	guint        update_id;
 };
 
 enum {
+	COVER_CLICKED,
 	SKIP_TO,
         LAST_SIGNAL
 };
@@ -113,6 +118,15 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
 
         parent_class = g_type_class_peek_parent (class);
 
+	goo_player_info_signals[COVER_CLICKED] =
+                g_signal_new ("cover_clicked",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (GooPlayerInfoClass, cover_clicked),
+			      NULL, NULL,
+			      goo_marshal_VOID__VOID,
+			      G_TYPE_NONE, 
+			      0);
 	goo_player_info_signals[SKIP_TO] =
                 g_signal_new ("skip_to",
 			      G_TYPE_FROM_CLASS (class),
@@ -127,7 +141,7 @@ goo_player_info_class_init (GooPlayerInfoClass *class)
 	gobject_class = G_OBJECT_CLASS (class);
         gobject_class->finalize = goo_player_info_finalize;
 
-	widget_class = (GtkWidgetClass*) class;
+	widget_class = GTK_WIDGET_CLASS (class);
 	widget_class->size_request = goo_player_info_size_request;
 }
 
@@ -137,8 +151,18 @@ set_label (GtkWidget     *label,
 	   const char    *format,
 	   const char    *text)
 {
+	char *e_text;
 	char *markup;
-	markup = g_strdup_printf (format, text);
+
+	if ((text == NULL) || (*text == '\0')) {
+		gtk_label_set_text (GTK_LABEL (label), "");
+		return;
+	}
+
+	e_text = g_markup_escape_text (text, -1);
+	markup = g_strdup_printf (format, e_text);
+	g_free (e_text);
+	
 	gtk_label_set_markup (GTK_LABEL (label), markup);
 	g_free (markup);
 }
@@ -190,25 +214,83 @@ time_scale_value_changed_cb (GtkRange      *range,
 {
 	double new_value = gtk_range_get_value (range);
 	int    seconds = (int) (new_value * info->priv->song_length);
+
+	if (info->priv->dragging) {
+		GooPlayerInfoPrivateData *priv = info->priv;
+		gint64 current_time;
+
+		current_time = priv->song_length * (new_value / 100.0);
+
+		set_time_string (priv->current_time, current_time);
+		sprintf (priv->time, _("%s / %s"), priv->total_time, priv->current_time);
+		set_time (info, priv->time);
+
+		return;
+	}
+
 	g_signal_emit (info, goo_player_info_signals[SKIP_TO], 0, seconds);
 }
 
 
-static void
-time_scale_enter_notify_cb (GtkRange         *range,
-			    GdkEventCrossing *event,
+static gboolean
+update_time_label_cb (gpointer data)
+{
+	GooPlayerInfo *info = data;
+	GooPlayerInfoPrivateData *priv = info->priv;
+	double new_value = gtk_range_get_value (GTK_RANGE (priv->time_scale));
+	gint64 current_time;
+
+	if (priv->update_id != 0) {
+		g_source_remove (priv->update_id);
+		priv->update_id = 0;
+	}
+	
+	current_time = priv->song_length * new_value;
+	set_time_string (priv->current_time, current_time);
+	sprintf (priv->time, _("%s / %s"), priv->current_time, priv->total_time);
+	set_time (info, priv->time);
+
+	priv->update_id = g_timeout_add (UPDATE_TIMEOUT,
+					 update_time_label_cb,
+					 data);
+
+	return FALSE;
+}
+
+
+static gboolean
+time_scale_button_press_cb (GtkRange         *range,
+			    GdkEventButton   *event,
 			    GooPlayerInfo    *info)
 {
-	info->priv->update_slider = FALSE;
+	info->priv->dragging = TRUE;
+	info->priv->update_id = g_timeout_add (UPDATE_TIMEOUT,
+					       update_time_label_cb,
+					       info);
+	return FALSE;
+}
+
+
+static gboolean
+time_scale_button_release_cb (GtkRange         *range,
+			      GdkEventButton   *event,
+			      GooPlayerInfo    *info)
+{
+	if (info->priv->update_id != 0) {
+		g_source_remove (info->priv->update_id);
+		info->priv->update_id = 0;
+	}
+
+	info->priv->dragging = FALSE;
+	return FALSE;
 }
 
 
 static void
-time_scale_leave_notify_cb (GtkRange         *range,
-			    GdkEventCrossing *event,
-			    GooPlayerInfo    *info)
+cover_button_clicked_cb (GtkWidget     *button,
+			 GooPlayerInfo *info)
 {
-	info->priv->update_slider = TRUE;
+	g_signal_emit (info, goo_player_info_signals[COVER_CLICKED], 0);
 }
 
 
@@ -221,13 +303,18 @@ goo_player_info_init (GooPlayerInfo *info)
 	info->priv = g_new0 (GooPlayerInfoPrivateData, 1);
 	priv = info->priv;
 
-	priv->update_slider = TRUE;
+	priv->dragging = FALSE;
 	priv->current_time[0] = '\0';
 	priv->total_time[0] = '\0';
+	priv->update_id = 0;
 
 	GTK_BOX (info)->spacing = SPACING;
 	GTK_BOX (info)->homogeneous = FALSE;
 
+	priv->tips = gtk_tooltips_new ();
+	gtk_object_ref (GTK_OBJECT (priv->tips));
+	gtk_object_sink (GTK_OBJECT (priv->tips));
+
 	/* Title and Artist */
 
 	priv->title1_label = gtk_label_new (NULL);
@@ -241,14 +328,10 @@ goo_player_info_init (GooPlayerInfo *info)
 	gtk_misc_set_alignment (GTK_MISC (priv->title3_label), 0.0, 0.5);
 	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), TRUE);
 
-	/* time */
+	/* Time */
 
 	time_box = gtk_hbox_new (FALSE, 6);
 
-	priv->time_label = gtk_label_new (NULL);
-	/*gtk_misc_set_alignment (GTK_MISC (priv->time_label), 0.0, 0.5);*/
-	gtk_widget_set_no_show_all (priv->time_label, TRUE);
-
 	priv->time_scale = gtk_hscale_new_with_range (0.0, 1.0, 0.01);
 	gtk_range_set_increments (GTK_RANGE (priv->time_scale), 0.01, 0.1);
 	gtk_scale_set_draw_value (GTK_SCALE (priv->time_scale), FALSE);
@@ -256,28 +339,47 @@ goo_player_info_init (GooPlayerInfo *info)
 	gtk_range_set_update_policy (GTK_RANGE (priv->time_scale), GTK_UPDATE_DISCONTINUOUS);
 	gtk_widget_set_no_show_all (priv->time_scale, TRUE);
 
+	priv->time_label = gtk_label_new (NULL);
+	gtk_widget_set_no_show_all (priv->time_label, TRUE);
+
 	priv->playing_label = gtk_label_new (NULL);
-	/*gtk_misc_set_alignment (GTK_MISC (priv->playing_label), 0.0, 0.0);*/
 	gtk_widget_set_no_show_all (priv->playing_label, TRUE);
 
-	gtk_box_pack_start (GTK_BOX (time_box), priv->time_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (time_box), priv->time_scale, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (time_box), priv->time_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (time_box), priv->playing_label, FALSE, FALSE, 0);
 
 	/* Image */
 
-	priv->image = gtk_image_new_from_stock (GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
-	gtk_widget_set_size_request (priv->image, COVER_SIZE, COVER_SIZE);
-	gtk_widget_show (priv->image);
-	gtk_box_pack_start (GTK_BOX (info), priv->image, FALSE, FALSE, 0);
+	priv->cover_button = gtk_button_new ();
+	gtk_button_set_relief (GTK_BUTTON (priv->cover_button),
+			       GTK_RELIEF_NONE);
+	gtk_tooltips_set_tip (priv->tips,
+			      GTK_WIDGET (priv->cover_button),
+			      _("Click here to choose a cover for this CD"),
+			      NULL);
+	g_signal_connect (G_OBJECT (priv->cover_button),
+			  "clicked",
+			  G_CALLBACK (cover_button_clicked_cb),
+			  info);
 
+	priv->cover_image = gtk_image_new_from_stock (GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
+	gtk_widget_set_size_request (priv->cover_image, COVER_SIZE, COVER_SIZE);
+	gtk_widget_show (priv->cover_image);
+	gtk_container_add (GTK_CONTAINER (priv->cover_button), priv->cover_image);
+
+	gtk_box_pack_start (GTK_BOX (info), priv->cover_button, FALSE, FALSE, 0);
+
+	/*FIXME
 	gtk_box_pack_start (GTK_BOX (info), 
 			    gtk_vseparator_new (),
-			    FALSE, FALSE, 6);
+			    FALSE, FALSE, 0);
+	*/
 
 	/**/
 
 	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 	gtk_box_pack_start (GTK_BOX (vbox), priv->title1_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), priv->title2_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), priv->title3_label, FALSE, FALSE, 0);
@@ -297,12 +399,12 @@ goo_player_info_init (GooPlayerInfo *info)
 			  G_CALLBACK (time_scale_value_changed_cb), 
 			  info);
 	g_signal_connect (priv->time_scale, 
-			  "enter-notify-event",
-			  G_CALLBACK (time_scale_enter_notify_cb), 
+			  "button-press-event",
+			  G_CALLBACK (time_scale_button_press_cb), 
 			  info);
 	g_signal_connect (priv->time_scale, 
-			  "leave-notify-event",
-			  G_CALLBACK (time_scale_leave_notify_cb), 
+			  "button_release_event",
+			  G_CALLBACK (time_scale_button_release_cb), 
 			  info);
 }
 
@@ -317,8 +419,13 @@ goo_player_info_finalize (GObject *object)
   
 	info = GOO_PLAYER_INFO (object);
 	if (info->priv != NULL) {
+		if (info->priv->update_id != 0) {
+			g_source_remove (info->priv->update_id);
+			info->priv->update_id = 0;
+		}
 		if (info->priv->player != NULL)
 			g_object_unref (info->priv->player);
+		gtk_object_unref (GTK_OBJECT (info->priv->tips));
 		g_free (info->priv);
 		info->priv = NULL;
 	}
@@ -443,8 +550,12 @@ goo_player_info_update_state (GooPlayerInfo  *info)
 		} else if (state == GOO_PLAYER_STATE_SEEKING) {
 			set_title1 (info, _("Reading CD"));
 			set_title2 (info, "");
-			
-		}  else {
+
+		} else if (state == GOO_PLAYER_STATE_LISTING) {
+			set_title1 (info, _("Reading CD"));
+			set_title2 (info, "");
+
+		} else {
 			const char *title, *subtitle;
 
 			title = goo_player_get_title (priv->player);
@@ -486,13 +597,33 @@ goo_player_info_set_time (GooPlayerInfo  *info,
 {
 	GooPlayerInfoPrivateData *priv = info->priv;
 
+	if (priv->dragging) 
+		return;
+
 	set_time_string (priv->current_time, current_time);
-	sprintf (priv->time, "%s", priv->current_time);
+	sprintf (priv->time, _("%s / %s"), priv->current_time, priv->total_time);
 	set_time (info, priv->time);
 
-	if (priv->update_slider) {
-		g_signal_handlers_block_by_data (priv->time_scale, info);
-		gtk_range_set_value (GTK_RANGE (priv->time_scale), (double) current_time / priv->song_length);
-		g_signal_handlers_unblock_by_data (priv->time_scale, info);
-	}
+	g_signal_handlers_block_by_data (priv->time_scale, info);
+	gtk_range_set_value (GTK_RANGE (priv->time_scale), (double) current_time / priv->song_length);
+	g_signal_handlers_unblock_by_data (priv->time_scale, info);
+}
+
+
+void
+goo_player_info_set_sensitive (GooPlayerInfo  *info,
+			       gboolean        value)
+{
+	gtk_widget_set_sensitive (info->priv->cover_button, value);
+}
+
+
+void
+goo_player_info_set_cover (GooPlayerInfo  *info,
+			   GdkPixbuf      *cover)
+{
+	if (cover != NULL)
+		gtk_image_set_from_pixbuf (GTK_IMAGE (info->priv->cover_image), cover);
+	else
+		gtk_image_set_from_stock (GTK_IMAGE (info->priv->cover_image), GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
 }
