@@ -62,12 +62,13 @@
 #define FILES_TO_PROCESS_AT_ONCE 500
 #define DEFAULT_WIN_WIDTH 200
 #define DEFAULT_WIN_HEIGHT 400
-#define HIDE_TRACK_LIST "Hide _tracks"
-#define SHOW_TRACK_LIST "Show _tracks"
+#define HIDE_TRACK_LIST N_("Hide _tracks")
+#define SHOW_TRACK_LIST N_("Show _tracks")
 #define DEFAULT_DEVICE "/dev/cdrom"
 #define DEFAULT_VOLUME 100
 #define PLAYER_CHECK_RATE 100
 #define COVER_SIZE 80
+#define IDLE_TIMEOUT 200
 
 struct _GooWindowPrivateData {
 	GtkUIManager    *ui;
@@ -108,7 +109,7 @@ struct _GooWindowPrivateData {
 
 	GooPlayer       *player;
 	GList           *song_list;                 /* SongInfo list. */
-	int              songs;
+	int              songs, total_time;
 	SongInfo        *current_song;
 	GList           *playlist;                  /* int list. */
 
@@ -122,6 +123,8 @@ struct _GooWindowPrivateData {
 	GList           *url_list;
 
 	GtkWidget       *preview;
+
+	int              pos_x, pos_y;
 };
 
 static int icon_size = 0;
@@ -166,40 +169,47 @@ static void
 window_update_statusbar_list_info (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
-	GList *scan;
-	int    tot_songs = 0, tot_time = 0;
 
 	if (window == NULL)
 		return;
 
 	gtk_statusbar_pop (GTK_STATUSBAR (priv->statusbar), priv->list_info_cid);
 
-	for (scan = priv->song_list; scan; scan = scan->next) {
-		SongInfo *song = scan->data;
-		tot_songs++;
-		tot_time += song->time;
-		goo_player_info_set_total_time (GOO_PLAYER_INFO (priv->info), tot_time);
-	}
-
-	if (tot_songs != 0) {
+	if (priv->songs != 0) {
 		const char *album;
 		char        time_text[64];
-		char       *info = NULL;
+		char       *tracks_s = NULL;
 		char       *text;
+		int         year;
+		char       *year_s;
 
 		album = goo_player_cd_get_album (GOO_PLAYER_CD (priv->player));
 
-		set_time_string (time_text, tot_time);
-		info = g_strdup_printf (ngettext ("%d track, %s", "%d tracks, %s", tot_songs), tot_songs, time_text);
+		set_time_string (time_text, priv->total_time);
+		tracks_s = g_strdup_printf (ngettext ("%d track", "%d tracks", priv->songs), priv->songs);
 
+		year = goo_player_get_year (priv->player);
+		if (year > 0)
+			year_s = g_strdup_printf ("year %d", year);
+                else
+			year_s = g_strdup ("");
+		
 		if (album != NULL)
-			text = g_strconcat (album, ", ", info, NULL);
+			text = g_strconcat (album, 
+					    ", ", year_s, 
+					    ", ", tracks_s, 
+					    ", ", time_text, 
+					    NULL);
 		else
-			text = g_strconcat (info, NULL);
+			text = g_strconcat (year_s, 
+					    ", ", tracks_s, 
+					    ", ", time_text,
+					    NULL);
 
 		gtk_statusbar_push (GTK_STATUSBAR (priv->statusbar), priv->list_info_cid, text);
 
-		g_free (info);
+		g_free (year_s);
+		g_free (tracks_s);
 		g_free (text);
 	}
 }
@@ -356,6 +366,7 @@ goo_window_update_list (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
 	UpdateData *udata;
+	GList      *scan;
 
 	if (GTK_WIDGET_REALIZED (priv->list_view))
 		gtk_tree_view_scroll_to_point (GTK_TREE_VIEW (priv->list_view), 0, 0);
@@ -374,7 +385,15 @@ goo_window_update_list (GooWindow *window)
 	/**/
 
 	priv->song_list = goo_player_get_song_list (priv->player);
-	priv->songs = g_list_length (priv->song_list);
+	priv->songs = 0;
+	priv->total_time = 0;
+	for (scan = priv->song_list; scan; scan = scan->next) {
+		SongInfo *song = scan->data;
+		priv->songs++;
+		priv->total_time += song->time;
+	}
+
+	goo_player_info_set_total_time (GOO_PLAYER_INFO (priv->info), priv->total_time);
 	
 	/**/
 
@@ -851,10 +870,9 @@ goo_window_unrealize (GtkWidget *widget)
 	if (playlist_visible)
 		save_window_size (window);
 
-	/*
+	/* FIXME
 	preferences_set_sort_method (window->sort_method);
 	preferences_set_sort_type (window->sort_type);
-	preferences_set_list_mode (window->list_mode);
 	*/
 
 	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
@@ -891,8 +909,7 @@ goo_window_show (GtkWidget *widget)
 	goo_window_set_statusbar_visibility (window, view_foobar);
 
 	if (window->priv->first_timeout_handle == 0)
-		window->priv->first_timeout_handle = g_timeout_add (200, /*FIXME*/
-								    first_time_idle, window);
+		window->priv->first_timeout_handle = g_timeout_add (IDLE_TIMEOUT, first_time_idle, window);
 }
 
 
@@ -1108,8 +1125,6 @@ player_start_cb (GooPlayer       *player,
 
 	switch (action) {
 	case GOO_PLAYER_ACTION_PLAY:
-		goo_player_info_set_time (GOO_PLAYER_INFO (priv->info), 0);
-
 		set_action_label_and_icon (window,
 					   "TogglePlay", 
 					   _("_Pause"), 
@@ -1167,11 +1182,17 @@ goo_window_set_current_song (GooWindow *window,
 			     int        n)
 {
 	GooWindowPrivateData *priv = window->priv;
+	SongInfo *song;
+
 	if (priv->current_song != NULL) {
 		song_info_unref (priv->current_song);	
 		priv->current_song = NULL;
 	}
+
 	priv->current_song = goo_player_get_song (priv->player, n);
+
+	song = (SongInfo*) priv->current_song;
+	goo_player_info_set_total_time (GOO_PLAYER_INFO (priv->info), song->time);
 }
 
 
@@ -1278,6 +1299,18 @@ goo_window_update_cover (GooWindow *window)
 }
 
 
+static gboolean
+autoplay_cb (gpointer data)
+{
+	GooWindow *window = data;
+	
+	if (window->priv->songs > 0) 
+		goo_window_play (window);
+	
+	return FALSE;
+}
+
+
 static void
 player_done_cb (GooPlayer       *player,
 		GooPlayerAction  action,
@@ -1294,6 +1327,10 @@ player_done_cb (GooPlayer       *player,
 		window_update_title (window);
 		goo_window_update_list (window);
 		goo_window_update_cover (window);
+		if (AutoPlay) {
+			AutoPlay = FALSE;
+			g_idle_add (autoplay_cb, window);
+		}
 		break;
 	case GOO_PLAYER_ACTION_METADATA:
 		goo_player_info_update_state (GOO_PLAYER_INFO (priv->info));
@@ -1315,8 +1352,7 @@ player_done_cb (GooPlayer       *player,
 					   "/MenuBar/CDMenu/",
 					   NULL);
 		if (action == GOO_PLAYER_ACTION_PLAY)
-			priv->next_timeout_handle = g_timeout_add (200, /*FIXME*/
-								   next_time_idle, window);
+			priv->next_timeout_handle = g_timeout_add (IDLE_TIMEOUT, next_time_idle, window);
 
 		break;
 	case GOO_PLAYER_ACTION_PAUSE:
@@ -2607,8 +2643,12 @@ goo_window_search_cover_on_internet (GooWindow *window)
 	album = goo_player_cd_get_album (GOO_PLAYER_CD (window->priv->player));
 	artist = goo_player_cd_get_artist (GOO_PLAYER_CD (window->priv->player));
 
-	if ((album == NULL) || (artist == NULL))
-		return; /*FIXME*/
+	if ((album == NULL) || (artist == NULL)) {
+		_gtk_error_dialog_run (GTK_WINDOW (window),
+				       _("Could not search a cover on Internet"),
+				       _("You have to enter the artist and album names in order to find the album cover."));
+		return;
+	}
 
 	dlg_cover_chooser (window, album, artist);
 }
@@ -2634,6 +2674,9 @@ void
 goo_window_toggle_visibility (GooWindow *window)
 {
 	if (GTK_WIDGET_VISIBLE (window)) {
+		gtk_window_get_position (GTK_WINDOW (window),
+					 &window->priv->pos_x,
+					 &window->priv->pos_y);
 		gtk_widget_hide (GTK_WIDGET (window));
 		set_action_label_and_icon (window,
 					   "ToggleVisibility", 
@@ -2645,7 +2688,10 @@ goo_window_toggle_visibility (GooWindow *window)
 					   NULL);
 
 	} else {
-		gtk_widget_show (GTK_WIDGET (window));
+		gtk_window_move (GTK_WINDOW (window),
+				 window->priv->pos_x,
+				 window->priv->pos_y);
+		gtk_window_present (GTK_WINDOW (window));
 		set_action_label_and_icon (window,
 					   "ToggleVisibility", 
 					   _("_Hide Window"), 
